﻿namespace TabsStudioNavigator
{
    class CommandsImpl
    {
        public CommandsImpl(TabsStudioExt.ITabsStudioEngine engine_, Options options_, bool useActivateTabFocusWorkaround)
        {
            options = options_;
            tabs = new Tabs(engine_, useActivateTabFocusWorkaround);
        }

        public void NavigateToNextTab()
        {
            if (tabs.Count() > 0)
            {
                Tab nextTab = OffsetSelectedTab(1);
                if (nextTab != null)
                    nextTab.Activate();
            }
        }

        public void NavigateToPreviousTab()
        {
            if (tabs.Count() > 0)
            {
                Tab nextTab = OffsetSelectedTab(-1);
                if (nextTab != null)
                    nextTab.Activate();
            }
        }

        public void NavigateToLastTab()
        {
            if (tabs.Count() > 0)
                tabs.GetTab(tabs.Count() - 1).Activate();
        }

        public void NavigateToNextWindow()
        {
            if (tabs.Count() > 0)
            {
                Tab tab = tabs.GetSelectedTab();
                uint i = tab.GetExtensionIndex(tab.GetActiveExtension());
                if (i + 1 < tab.Count())
                {
                    tab.ActivateExtension(tab.GetExtension(i + 1));
                }
                else
                {
                    Tab nextTab = OffsetSelectedTab(1);
                    if (nextTab != null)
                        nextTab.ActivateExtension(nextTab.GetExtension(0));
                }
            }
        }

        public void NavigateToPreviousWindow()
        {
            if (tabs.Count() > 0)
            {
                Tab tab = tabs.GetSelectedTab();
                uint i = tab.GetExtensionIndex(tab.GetActiveExtension());
                if (i >= 1)
                {
                    tab.ActivateExtension(tab.GetExtension(i - 1));
                }
                else
                {
                    Tab nextTab = OffsetSelectedTab(-1);
                    if (nextTab != null)
                        nextTab.ActivateExtension(nextTab.GetExtension(nextTab.Count() - 1));
                }
            }
        }

        public void NavigateToNextGroup()
        {
            if (tabs.GroupCount() < 2)
                return;

            Group selected = tabs.GetSelectedGroup();
            if (selected == null)
                return;

            uint i = tabs.GetGroupIndex(selected);
            if (i == tabs.GroupCount() - 1)
            {
                if (options.cycleTab)
                    tabs.GetGroup(0).Activate();
                return;
            }

            tabs.GetGroup(++i).Activate();
        }

        public void NavigateToPreviousGroup()
        {
            if (tabs.GroupCount() < 2)
                return;

            Group selected = tabs.GetSelectedGroup();
            if (selected == null)
                return;

            uint i = tabs.GetGroupIndex(selected);
            if (i == 0)
            {
                if (options.cycleTab)
                    tabs.GetGroup(tabs.GroupCount() - 1).Activate();
                return;
            }

            tabs.GetGroup(--i).Activate();
        }

        public void MoveTabLeft()
        {
            if (tabs.Count() > 1)
            {
                Tab nextTab = OffsetSelectedTab(-1);
                if (nextTab != null)
                    tabs.MoveTab(tabs.GetSelectedTab(), nextTab);
            }
        }

        public void MoveTabRight()
        {
            if (tabs.Count() > 1)
            {
                Tab nextTab = OffsetSelectedTab(1);
                if (nextTab != null)
                    tabs.MoveTab(tabs.GetSelectedTab(), nextTab);
            }
        }

        // REQUIRE(i >= 1)
        public void NavigateToTab(int i)
        {
            uint index = (uint)i - 1;
            if (tabs.Count() > index)
                tabs.GetTab(index).Activate();
        }

        private Tab OffsetSelectedTab(int offset)
        {
            Tab selectedTab = tabs.GetSelectedTab();
            if (selectedTab == null)
                return null;
            int i = (int)tabs.GetTabIndex(selectedTab) + offset;
            if (options.cycleTab)
            {
                if (i >= tabs.Count())
                {
                    i = 0;
                }
                if (i < 0)
                {
                    i = (int)tabs.Count() - 1;
                }
            }
            if (i >= 0 && i < tabs.Count())
                return tabs.GetTab((uint)i);
            else
                return null;
        }

        private Options options;
        private Tabs tabs;
    }
}
